const readline = require('readline');

const CHOICES = ['rock', 'paper', 'scissors'];

const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
});

function getComputerChoice()
{
    const randomIndex = Math.floor(Math.random() * CHOICES.length);
    return CHOICES[randomIndex];
}

function determineWinner(player, computer)
{
    if (player === computer)
    {
        return 'tie';
    }

    if (
        (player === 'rock' && computer === 'scissors') ||
        (player === 'scissors' && computer === 'paper') ||
        (player === 'paper' && computer === 'rock')
    )
    {
        return 'player';
    }
    return 'computer';
}

function displayStatistics(stats)
{
    console.log('\n');
    console.log(`Wins: ${stats.wins}`);
    console.log(`Losses: ${stats.losses}`);
    console.log(`Ties: ${stats.ties}`);
    console.log('\n');
}

function playGame()
{
    console.log('Welcome to Rock, Paper, Scissors!');

    rl.question('Choose match format (3 for Best-of-3, 5 for Best-of-5): ', (format) => {
        const matchFormat = parseInt(format, 10);
        if (![3, 5].includes(matchFormat))
        {
            console.log('Invalid format! Please restart the game and enter 3 or 5.');
            rl.close();
            return;
        }

        const stats = { wins: 0, losses: 0, ties: 0 };
        const requiredWins = Math.ceil(matchFormat / 2);

        console.log('\n');
        console.log("Let's start the game!");
        console.log('\n');

        function playRound()
        {
            rl.question('Your move (rock, paper, scissors): ', (playerChoice) => {
                const sanitizedChoice = playerChoice.trim().toLowerCase();

                if (!CHOICES.includes(sanitizedChoice))
                {
                    console.log('Invalid choice! Please type rock, paper, or scissors.');
                    return playRound();
                }

                const computerChoice = getComputerChoice();
                console.log(`You chose: ${sanitizedChoice}`);
                console.log(`Computer chose: ${computerChoice}`);

                const winner = determineWinner(sanitizedChoice, computerChoice);

                if (winner === 'player')
                {
                    console.log('You win this round!');
                    stats.wins++;
                }
                else if (winner === 'computer')
                {
                    console.log('Computer wins this round!');
                    stats.losses++;
                }
                else
                {
                    console.log("It's a tie!");
                    stats.ties++;
                }

                displayStatistics(stats);

                if (stats.wins === matchFormat)
                {
                    console.log('Congratulations! You won the match!');
                    askToRestart();
                    // rl.close();
                }
                else if (stats.losses === matchFormat)
                {
                    console.log('The computer won the match. Better luck next time!');
                    askToRestart();
                    // rl.close();
                }
                else
                {
                    playRound();
                }
            });
        }

        function askToRestart()
        {
            rl.question('Do you want to play again? (yes/no): ', (answer) => {
                const sanitizedAnswer = answer.trim().toLowerCase();
                if (sanitizedAnswer === 'yes')
                {
                    playGame();
                }
                else if (sanitizedAnswer === 'no')
                {
                    console.log('Thanks for playing! Goodbye!');
                    rl.close();
                }
                else
                {
                    console.log('Invalid input. Please type "yes" or "no".');
                    askToRestart();
                }
            });
        }
        playRound();
    });
}
playGame();